$NetBSD$

- pull upstream changes necessary for mikutter_gtk3

--- lib/gobject-introspection/type-tag.rb.orig	2021-11-05 16:48:04.483066471 +0000
+++ lib/gobject-introspection/type-tag.rb
@@ -1,4 +1,4 @@
-# Copyright (C) 2019  Ruby-GNOME Project Team
+# Copyright (C) 2019-2021  Ruby-GNOME Project Team
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -17,171 +17,163 @@
 module GObjectIntrospection
   class TypeTag
     def try_convert(type_info, value)
-      nil
+      method = "try_convert_#{nick}"
+      if respond_to?(method, true)
+        __send__(method, type_info, value)
+      else
+        nil
+      end
     end
 
     def description(type_info)
-      nick
-    end
-
-    class << BOOLEAN
-      def try_convert(type_info, value)
-        case value
-        when true, false
-          value
-        when nil
-          false
+      method = "description_#{nick}"
+      if respond_to?(method, true)
+        __send__(method, type_info)
+      else
+        nick
+      end
+    end
+
+    private
+    def try_convert_boolean(type_info, value)
+      case value
+      when true, false
+        value
+      when nil
+        false
+      else
+        nil
+      end
+    end
+
+    def try_convert_integer(type_info, value)
+      if value.is_a?(Integer)
+        value
+      elsif value.respond_to?(:to_int)
+        value.to_int
+      else
+        nil
+      end
+    end
+
+    alias_method :try_convert_int8, :try_convert_integer
+    alias_method :try_convert_uint8, :try_convert_integer
+    alias_method :try_convert_int16, :try_convert_integer
+    alias_method :try_convert_uint16, :try_convert_integer
+    alias_method :try_convert_int32, :try_convert_integer
+    alias_method :try_convert_uint32, :try_convert_integer
+    alias_method :try_convert_int64, :try_convert_integer
+    alias_method :try_convert_uint64, :try_convert_integer
+
+    def try_convert_float(type_info, value)
+      if value.is_a?(Float)
+        value
+      elsif value.respond_to?(:to_f) # TODO: Should we stop this?
+        value.to_f
+      else
+        nil
+      end
+    end
+
+    alias_method :try_convert_double, :try_convert_float
+
+    def try_convert_gtype(type_info, value)
+      GLib::Type.try_convert(value)
+    end
+
+    def try_convert_utf8(type_info, value)
+      case value
+      when String
+        value.encode(Encoding::UTF_8)
+      when Symbol
+        value.to_s.encode(Encoding::UTF_8)
+      else
+        if value.respond_to?(:to_str)
+          value.to_str.encode(Encoding::UTF_8)
         else
           nil
         end
       end
     end
 
-    module IntegerTypeTag
-      def try_convert(type_info, value)
-        if value.is_a?(Integer)
-          value
-        elsif value.respond_to?(:to_int)
-          value.to_int
+    def try_convert_filename(type_info, value)
+      case value
+      when String
+        value.encode(GLib::FILENAME_ENCODING)
+      else
+        if value.respond_to?(:to_path)
+          value.to_path.encode(GLib::FILENAME_ENCODING)
+        elsif value.respond_to?(:to_str)
+          value.to_str.encode(GLib::FILENAME_ENCODING)
         else
           nil
         end
       end
     end
 
-    INT8.extend(IntegerTypeTag)
-    UINT8.extend(IntegerTypeTag)
-    INT16.extend(IntegerTypeTag)
-    UINT16.extend(IntegerTypeTag)
-    INT32.extend(IntegerTypeTag)
-    UINT32.extend(IntegerTypeTag)
-    INT64.extend(IntegerTypeTag)
-    UINT64.extend(IntegerTypeTag)
-
-    module FloatTypeTag
-      def try_convert(type_info, value)
-        if value.is_a?(Float)
-          value
-        elsif value.respond_to?(:to_f) # TODO: Should we stop this?
-          value.to_f
-        else
-          nil
+    def try_convert_array_like(type_info, value)
+      value = Array.try_convert(value)
+      return nil if value.nil?
+      element_type_info = get_element_type_info(type_info)
+      value.collect do |v|
+        unless v.nil?
+          v = element_type_info.try_convert(v)
+          return nil if v.nil?
         end
+        v
       end
     end
 
-    FLOAT.extend(FloatTypeTag)
-    DOUBLE.extend(FloatTypeTag)
-
-    class << GTYPE
-      def try_convert(type_info, value)
-        GLib::Type.try_convert(value)
-      end
+    def description_array_like(type_info)
+      element_type_info = type_info.get_param_type(0)
+      "#{nick}(#{element_type_info.description})"
     end
 
-    class << UTF8
-      def try_convert(type_info, value)
-        case value
-        when String
-          value.encode(Encoding::UTF_8)
-        when Symbol
-          value.to_s.encode(Encoding::UTF_8)
-        else
-          if value.respond_to?(:to_str)
-            value.to_str.encode(Encoding::UTF_8)
-          else
-            nil
-          end
-        end
-      end
+    def get_element_type_info(type_info)
+      type_info.get_param_type(0)
     end
 
-    class << FILENAME
-      def try_convert(type_info, value)
+    def try_convert_array(type_info, value)
+      case get_element_type_info(type_info).tag
+      when INT8, UINT8
         case value
         when String
-          value.encode(GLib::FILENAME_ENCODING)
-        else
-          if value.respond_to?(:to_path)
-            value.to_path.encode(GLib::FILENAME_ENCODING)
-          elsif value.respond_to?(:to_str)
-            value.to_str.encode(GLib::FILENAME_ENCODING)
-          else
-            nil
-          end
+          return value
+        when GLib::Bytes
+          return value.to_s
         end
       end
+      try_convert_array_like(type_info, value)
     end
 
-    module ArrayTypeTag
-      def try_convert(type_info, value)
-        value = Array.try_convert(value)
-        return nil if value.nil?
-        element_type_info = get_element_type_info(type_info)
-        value.collect do |v|
-          unless v.nil?
-            v = element_type_info.try_convert(v)
-            return nil if v.nil?
-          end
-          v
-        end
-      end
+    alias_method :description_array, :description_array_like
 
-      def description(type_info)
-        element_type_info = type_info.get_param_type(0)
-        "#{super}(#{element_type_info.description})"
-      end
+    alias_method :try_convert_glist, :try_convert_array_like
+    alias_method :description_glist, :description_array_like
+    alias_method :try_convert_gslist, :try_convert_array_like
+    alias_method :description_gslist, :description_array_like
 
-      def get_element_type_info(type_info)
-        type_info.get_param_type(0)
+    def try_convert_ghash(type_info, value)
+      case value
+      when Hash
+        value
+      else
+        nil
       end
     end
 
-    class << ARRAY
-      include ArrayTypeTag
-
-      def try_convert(type_info, value)
-        case get_element_type_info(type_info).tag
-        when INT8, UINT8
-          case value
-          when String
-            return value
-          when GLib::Bytes
-            return value.to_s
-          end
-        end
-        super
-      end
+    def description_ghash(type_info)
+      key_type = type_info.get_param_type(0)
+      value_type = type_info.get_param_type(1)
+      "#{nick}(#{key_type.description}->#{value_type.description})"
     end
 
-    GLIST.extend(ArrayTypeTag)
-    GSLIST.extend(ArrayTypeTag)
-
-    class << GHASH
-      def try_convert(type_info, value)
-        case value
-        when Hash
-          value
-        else
-          nil
-        end
-      end
-
-      def description(type_info)
-        key_type = type_info.get_param_type(0)
-        value_type = type_info.get_param_type(1)
-        "#{super}(#{key_type.description}->#{value_type.description})"
-      end
+    def try_convert_interface(type_info, value)
+      type_info.interface.try_convert(value)
     end
 
-    class << INTERFACE
-      def try_convert(type_info, value)
-        type_info.interface.try_convert(value)
-      end
-
-      def description(type_info)
-        "#{super}(#{type_info.interface.description})"
-      end
+    def description_interface(type_info)
+      "#{nick}(#{type_info.interface.description})"
     end
   end
 end
