$NetBSD$

- pull upstream changes necessary for mikutter_gtk3

--- lib/gdk3/loader.rb.orig	2021-11-05 16:55:37.043519247 +0000
+++ lib/gdk3/loader.rb
@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2018  Ruby-GNOME2 Project Team
+# Copyright (C) 2013-2021  Ruby-GNOME Project Team
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -22,34 +22,83 @@ module Gdk
     end
 
     private
-    def window_class
-      @window_class ||= @base_module.const_get(:Window)
-    end
-
     def event_class
       @event_class ||= @base_module.const_get(:Event)
     end
 
-    def event_motion_class
-      @event_motion_class ||= @base_module.const_get(:EventMotion)
-    end
-
     def pre_load(repository, namespace)
       setup_pending_constants
       setup_pending_rectangle_functions
       define_keyval_module
       define_selection_module
+      define_event_singleton_methods_module
+      define_event_motion_methods_module
+      define_window_methods_module
+      define_cairo_context_methods_module
+      define_cairo_surface_methods_module
       require_pre_libraries
     end
 
     def define_keyval_module
-      @keyval_module = Module.new
-      @base_module.const_set("Keyval", @keyval_module)
+      @keyval_module = define_methods_module(:Keyval)
+    end
+
+    def post_keyval_module
+      post_methods_module(@keyval_module)
     end
 
     def define_selection_module
       @selection_module = Module.new
-      @base_module.const_set("Selection", @selection_module)
+      @base_module.const_set(:Selection, @selection_module)
+    end
+
+    def define_event_singleton_methods_module
+      @event_singleton_methods_module =
+        define_methods_module(:EventSingletonMethods)
+    end
+
+    def apply_event_singleton_methods
+      event_class = @base_module.const_get(:Event)
+      apply_methods_module(@event_singleton_methods_module,
+                           event_class.singleton_class)
+    end
+
+    def define_event_motion_methods_module
+      @event_motion_methods_module =
+        define_methods_module(:EventMotionMethods)
+    end
+
+    def apply_event_motion_methods
+      event_motion_class = @base_module.const_get(:EventMotion)
+      apply_methods_module(@event_motion_methods_module,
+                           event_motion_class)
+    end
+
+    def define_window_methods_module
+      @window_methods_module = define_methods_module(:WindowMethods)
+    end
+
+    def apply_window_methods
+      window_class = @base_module.const_get(:Window)
+      apply_methods_module(@window_methods_module, window_class)
+    end
+
+    def define_cairo_context_methods_module
+      @cairo_context_methods_module =
+        define_methods_module(:CairoContextMethods)
+    end
+
+    def apply_cairo_context_methods
+      apply_methods_module(@cairo_context_methods_module, Cairo::Context)
+    end
+
+    def define_cairo_surface_methods_module
+      @cairo_surface_methods_module =
+        define_methods_module(:CairoSurfaceMethods)
+    end
+
+    def apply_cairo_surface_methods
+      apply_methods_module(@cairo_surface_methods_module, Cairo::Surface)
     end
 
     def require_pre_libraries
@@ -59,6 +108,12 @@ module Gdk
     def post_load(repository, namespace)
       apply_pending_constants
       apply_pending_rectangle_functions
+      post_keyval_module
+      apply_event_singleton_methods
+      apply_event_motion_methods
+      apply_window_methods
+      apply_cairo_context_methods
+      apply_cairo_surface_methods
       require_post_libraries
       convert_event_classes
       define_selection_constants
@@ -217,9 +272,9 @@ module Gdk
         target_class = nil
         case $POSTMATCH
         when "get_from_window"
-          target_class = window_class
+          target_class = @window_methods_module
         when "get_from_surface"
-          target_class = Cairo::Surface
+          target_class = @cairo_surface_methods_module
         end
         if target_class
           define_method(info, target_class, "to_pixbuf")
@@ -230,30 +285,38 @@ module Gdk
         name = $POSTMATCH
         case name
         when "request_motions"
-          define_method(info, event_motion_class, "request")
+          define_method(info, @event_motion_methods_module, "request")
         else
           # ignore because moved to Gdk::Event
         end
       when /\Aevents_/
         method_name = rubyish_method_name(info, :prefix => "events_")
-        define_singleton_method(event_class, method_name, info)
+        define_module_function(@event_singleton_methods_module,
+                               method_name,
+                               info)
       when /\Acairo_/
         name = $POSTMATCH
         case name
         when "create"
-          define_method(info, window_class, "create_cairo_context")
+          define_method(info, @window_methods_module, "create_cairo_context")
         when "set_source_color"
-          define_method(info, Cairo::Context, "set_source_gdk_color")
+          define_method(info,
+                        @cairo_context_methods_module,
+                        "set_source_gdk_color")
         when "set_source_rgba"
-          define_method(info, Cairo::Context, "set_source_gdk_rgba")
+          define_method(info,
+                        @cairo_context_methods_module,
+                        "set_source_gdk_rgba")
         when "rectangle"
-          define_method(info, Cairo::Context, "gdk_rectangle")
+          define_method(info,
+                        @cairo_context_methods_module,
+                        "gdk_rectangle")
         when "region_create_from_surface"
           # TODO
         when "surface_create_from_pixbuf"
           # TODO
         else
-          define_method(info, Cairo::Context, name)
+          define_method(info, @cairo_context_methods_module, name)
         end
       when /\Akeyval_/
         name = rubyish_method_name(info, :prefix => "keyval_")
